package net.ericaro.neobin;

/** contains the whole reflextion information about a binary type.
 * 
 * 
 * @author eric
 *
 */
public class BinaryType {

	private String javaName; // actual javaName (for native types, or object name ) ( int, long String etc.)
	private String javaVar; // prefered java var name (i for int etc.)
	private String name; // type name (why ?) ( Int, Long, String)
	private int width; // byte width
	private BinaryType component = null;// null by default unless composite is true

	/** build a composite binarytype, i.e an array of "components"
	 * 
	 * @param component
	 */
	BinaryType(BinaryType component) {
		this("java.nio." + component.getName() + "Buffer", component.getName() + "Buffer",component.getJavaVar() + "Buffer", -1);
		this.component = component;

	}

	/** build a primitive type.
	 * For instance for integer this would be
	 * <code>BinaryType("int", "Int", "i", 4 )</code>
	 * 
	 * the second "Int" is used to internally retrieve a buffer, hence there must be
	 * an IntBuffer class.
	 * 
	 * @param javaName
	 * @param name
	 * @param javaVar
	 * @param width
	 */
	BinaryType(String javaName, String name, String javaVar, int width) {
		super();
		this.javaName = javaName;
		this.name = name;
		this.width = width;
		this.javaVar = javaVar;
	}

	/** derive a BinaryType and changes only its name.
	 * 
	 * @param bt
	 * @param varName
	 */
	BinaryType(BinaryType bt, String varName) {
		this.javaName   = bt.javaName;
		this.javaVar    = varName;
		this.name 	    = bt.name;
		this.width 		= bt.width;
		this.component  = bt.component;
	}

	public String getJavaName() {
		return javaName;
	}

	public String getName() {
		return name;
	}

	public int getWidth() {
		return width;
	}

	public boolean isComposite() {
		return component != null;
	}

	public String getJavaVar() {
		return javaVar;
	}

	public BinaryType getComponent() {
		return component;
	}
	public boolean isByte() {
		return "byte".equals(javaName );
	}
}
